------------------------------------------------------
-- TrackMenu.lua
------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("GFW_TrackMenu");

local SpellIconPrefix = "Interface\\Icons\\";
local TrackingAbilities = {
	{ tab=1, icon="INV_Misc_Flower_02" },                               -- Find Herbs	
	{ tab=1, icon="Spell_Nature_Earthquake" },                          -- Find Minerals
	{ tab=1, icon="Racial_Dwarf_FindTreasure" },                        -- Find Treasure

	{ class="HUNTER", tab=4, icon="Ability_Tracking" },                 -- Track Beasts	
	{ class="HUNTER", tab=4, icon="Spell_Shadow_SummonFelHunter" },     -- Track Demons	
	{ class="HUNTER", tab=4, icon="INV_Misc_Head_Dragon_01" },          -- Track Dragonkin
	{ class="HUNTER", tab=4, icon="Spell_Frost_SummonWaterElemental" },	-- Track Elementals
	{ class="HUNTER", tab=4, icon="Ability_Racial_Avatar" },            -- Track Giants	
	{ class="HUNTER", tab=4, icon="Ability_Stealth" },                  -- Track Hidden	
	{ class="HUNTER", tab=4, icon="Spell_Holy_PrayerOfHealing" },       -- Track Humanoids
	{ class="HUNTER", tab=4, icon="Spell_Shadow_DarkSummoning" },       -- Track Undead	

	{ class="DRUID", tab=3, icon="Ability_Tracking" },   		-- Track Humanoids	

	{ class="WARLOCK", tab=3, icon="Spell_Shadow_Metamorphosis" },      -- Sense Demons	
	{ class="PALADIN", tab=2, icon="Spell_Holy_SenseUndead" },	        -- Sense Undead	
};

local options = { 
    type = 'group',
    args = {
		open = {
			type = 'text',
			name = L["Open Menu on"],
			desc = L["How to open the menu from the minimap tracking button."],
			get = "GetOpen",
			set = "SetOpen",
			validate = {"over", "click"},
		},
		anchor = {
			type = 'text',
			name = L["Menu Anchor"],
			desc = L["Which side of the button the menu appears on."],
			get = "GetAnchor",
			set = "SetAnchor",
			validate = {"top", "bottom", "left", "right"},
		},
		direction = {
			type = 'text',
			name = L["Menu Direction"],
			desc = L["Direction the menu grows in."],
			get = "GetDirection",
			set = "SetDirection",
			validate = {"up", "down"},
		},
		tooltip = {
			type = 'toggle',
			name = L["Show Tooltip"],
			desc = L["Whether to show a tooltip when mousing over the button (only when menu opens on click)."],
			get = "GetTooltip",
			set = "SetTooltip",
		},
	},
};
local defaults = {
    open = "click",
    direction = "down",
    anchor = "bottom",
	tooltip = true,
};

------------------------------------------------------
-- Addon setup
------------------------------------------------------


GFW_TrackMenu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0");
GFW_TrackMenu:RegisterChatCommand(L["Slash-Commands"], options);
local G = GFW_TrackMenu;

GFW_TrackMenu:RegisterDB("GFW_TrackMenu_DB", "GFW_TrackMenu_DBPC");
GFW_TrackMenu:RegisterDefaults("profile", defaults);

local dewdrop = AceLibrary("Dewdrop-2.0");


function GFW_TrackMenu:OnInitialize()
    -- Called when the addon is loaded
end

function GFW_TrackMenu:OnEnable()
	--self:Print("Enabled")

	-- Register general events
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "SetUpdateNeeded");
    self:RegisterEvent("PLAYER_REGEN_ENABLED", "SetUpdateNeeded");
    self:RegisterEvent("SPELLS_CHANGED", "SetUpdateNeeded");
    self:RegisterEvent("PLAYER_AURAS_CHANGED");
    self:RegisterEvent("DISPLAY_SIZE_CHANGED");
		
end

function GFW_TrackMenu:OnDisable()
    -- Called when the addon is disabled
end

------------------------------------------------------
-- Config
------------------------------------------------------
 
function GFW_TrackMenu:OnButtonEnter() 
	local state = G.RootFrame:GetAttribute("state");
	if (G.db.profile.open == "click" and G.db.profile.tooltip and tonumber(state) ~= 2) then	
		GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT");
		GameTooltip:SetTrackingSpell();
		GameTooltip:AddLine("<"..L["Shift-Click for TrackMenu Options"]..">", 0.5, 0.5, 0.5);
		GameTooltip:Show(); 	
	end
end 
 
function GFW_TrackMenu:OnButtonLeave() 
    this.updateTooltip = nil; 
    GameTooltip:Hide(); 
end 

function GFW_TrackMenu:GetOpen()
    return self.db.profile.open;
end

function GFW_TrackMenu:SetOpen(newValue)
    self.db.profile.open = newValue;
	GFW_TrackMenu:SetupMenuOpenAction();
end

function GFW_TrackMenu:GetDirection()
    return self.db.profile.direction;
end

function GFW_TrackMenu:SetDirection(newValue)
    self.db.profile.direction = newValue;
	self:ScheduleLeaveCombatAction("SetupSpellButtons");
end

function GFW_TrackMenu:GetAnchor()
    return self.db.profile.anchor;
end

function GFW_TrackMenu:SetAnchor(newValue)
    self.db.profile.anchor = newValue;
	self:ScheduleLeaveCombatAction("SetupSpellButtons");
end

function GFW_TrackMenu:GetTooltip()
    return self.db.profile.tooltip;
end

function GFW_TrackMenu:SetTooltip(newValue)
    self.db.profile.tooltip = newValue;
	self:ScheduleLeaveCombatAction("SetupSpellButtons");
end

function GFW_TrackMenu:OpenConfigMenu()
	GameTooltip:Hide(); 
	dewdrop:Open(this,
	    'children', function(level)
			if (level == 1) then
				dewdrop:AddLine('text', self:ToString(), 'isTitle', true);
			end
	        dewdrop:FeedAceOptionsTable(options);
			if (level == 1) then
				dewdrop:AddLine();
				dewdrop:AddLine('text', CLOSE, 'func', function() dewdrop:Close(); end);
			end
	    end
	);
end

------------------------------------------------------
-- Event handlers
------------------------------------------------------

function GFW_TrackMenu:PLAYER_AURAS_CHANGED()
	self:ScheduleLeaveCombatAction("CheckButtonVisibility");
end

function GFW_TrackMenu:DISPLAY_SIZE_CHANGED()
	self:ScheduleLeaveCombatAction("CheckButtonVisibility");
end

function GFW_TrackMenu:SetUpdateNeeded()
	self:ScheduleLeaveCombatAction("BuildSpellList");
	self:ScheduleLeaveCombatAction("SetupSpellButtons");
	self:ScheduleLeaveCombatAction("CheckButtonVisibility");
end

------------------------------------------------------
-- Building & running the menu
------------------------------------------------------

function GFW_TrackMenu:BuildSpellList()
	G.MyTrackSpells = {};
	G.SpellIcons = {};
	local _, myClass = UnitClass("player");
	for _, spellInfo in ipairs(TrackingAbilities) do
		--DevTools_Dump(spellInfo)
		if (spellInfo.class == myClass or not spellInfo.class) then
			local name, _, offset, numSpells = GetSpellTabInfo(spellInfo.tab);
			--DevTools_Dump({name=name})
			for i = offset + 1, offset + numSpells do
				local icon = GetSpellTexture(i, BOOKTYPE_SPELL);
				--DevTools_Dump(icon)
				if (icon == SpellIconPrefix..spellInfo.icon) then
					local spellName = GetSpellName(i, BOOKTYPE_SPELL);
					if (spellInfo.form) then
						local formIcon, formName, active, castable = GetShapeshiftFormInfo(spellInfo.form);
						if (active) then
							table.insert(G.MyTrackSpells, spellName);
							G.SpellIcons[spellName] = icon;
						end
					else
						table.insert(G.MyTrackSpells, spellName);
						G.SpellIcons[spellName] = icon;
					end
				end
			end
		end
	end
	--DevTools_Dump({G.MyTrackSpells, G.SpellIcons})
end

function GFW_TrackMenu:SetupSpellButtons()
	
	local numSpells = #G.MyTrackSpells;
	if (numSpells == 0 and not G.RootFrame) then
		-- don't pollute UI memory with frames if there's no tracking skills
		-- but if we've already made them, continue so we can clean them up
		return;	
	elseif (not G.RootFrame) then
		GFW_TrackMenu:BuildMainFrames();
	end
	
	local buttonHeight;
	if (self.db.profile.direction == "up") then
		buttonHeight = 28;
	else							-- down
		buttonHeight = -28;
	end
    local spellCount = math.max(#G.SpellButtons, numSpells);
    for i = 1, spellCount do
		local spellButton = GFW_TrackMenu:GetSpellButton(i);
		if (i > numSpells) then
		    spellButton:SetAttribute("showstates", nil);
		    spellButton:SetAttribute("hidestates", "*");
		else
			local yOffset;
			if (self.db.profile.anchor == "left" or self.db.profile.anchor == "right") then
				yOffset = (i - 1) * buttonHeight - 2;
			else
				yOffset = i * buttonHeight;
			end
			local xOffset;
			if (self.db.profile.anchor == "left") then
				xOffset = 6 - spellButton:GetWidth();
			elseif (self.db.profile.anchor == "right") then
				xOffset = G.HeaderButton:GetWidth() - 3;
			else
				xOffset = 3;
			end
			
			local spellname = G.MyTrackSpells[i];
		    spellButton:SetAttribute("spell", spellname);
		    spellButton:SetAttribute("showstates", "2");
		    spellButton:SetAttribute("hidestates", "!2");
		    spellButton:SetAttribute("ofsx", xOffset);
		    spellButton:SetAttribute("ofsy", yOffset);
		    spellButton:SetAttribute("ofspoint", "LEFT");			
		    spellButton:SetAttribute("ofsrelpoint", "LEFT");			
		    spellButton:SetAttribute("type", "spell");
		    spellButton:SetAttribute("newstate", "1");
			
			spellButton.texture:SetTexture(G.SpellIcons[spellname]);
			spellButton.texture:Show();
			spellButton.title:SetText(spellname);
			
		end 
    end
    
	if (numSpells > 0) then
		G.RootFrame:SetAttribute("state", 1);
	end
end

function GFW_TrackMenu:CheckButtonVisibility()
	if (not G.RootFrame) then return; end -- don't need to do anything if we haven't made our menu yet
	
	G.RootFrame:SetAllPoints(MiniMapTrackingFrame);
	G.RootFrame:SetFrameLevel(MiniMapTrackingFrame:GetFrameLevel() + 1);
	if (#G.MyTrackSpells == 0) then 
		FTM_NoTrackingFrame:Hide();
		G.RootFrame:SetAttribute("state", 0);
		return;
	end
	local icon = GetTrackingTexture();
	if (icon) then
		FTM_NoTrackingFrame:Hide();
	else
		FTM_NoTrackingFrame:Show();
	end
end

function GFW_TrackMenu:BuildMainFrames()
	G.RootFrame = CreateFrame("Frame", nil, Minimap, "SecureStateHeaderTemplate");
	G.RootFrame:SetAllPoints(MiniMapTrackingFrame);
	G.RootFrame:SetWidth(33);
	G.RootFrame:SetHeight(33);
	G.RootFrame:SetFrameStrata("BACKGROUND");
	G.RootFrame:SetFrameLevel(MiniMapTrackingFrame:GetFrameLevel() + 1);
	G.RootFrame:Show();

	G.RootFrame:SetAttribute("statemap-anchor-enter", nil);
	G.RootFrame:SetAttribute("statemap-anchor-leave",      ";"); -- Non-empty
	G.RootFrame:SetAttribute("delaystatemap-anchor-leave", "2:1");
	G.RootFrame:SetAttribute("delaytimemap-anchor-leave",  "2:0.5");
	G.RootFrame:SetAttribute("delayhovermap-anchor-leave", "2:true");

	G.HeaderButton = CreateFrame("Button", nil, nil, "FTM_HeaderTemplate,SecureAnchorEnterTemplate");
	G.RootFrame:SetFrameStrata("DIALOG");
	G.HeaderButton:SetWidth(33);
	G.HeaderButton:SetHeight(33);

	-- To make G.HeaderButton visible for debug
	-- G.HeaderButton.texture = G.HeaderButton:CreateTexture(nil, "OVERLAY");
	-- G.HeaderButton.texture:SetAllPoints(G.HeaderButton);
	-- G.HeaderButton.texture:SetTexture(0,1,0,0.5);
	-- G.HeaderButton.texture:Show();

	G.RootFrame:SetAttribute("addchild", G.HeaderButton);
	G.HeaderButton:SetAttribute("showstates", "!0,*");
	G.HeaderButton:SetAttribute("ofsx", 0);
	G.HeaderButton:SetAttribute("ofsy", 0);

	G.HeaderButton:SetAttribute("anchorchild", G.RootFrame);
	G.HeaderButton:SetAttribute("childstate", 2);
	G.HeaderButton:SetAttribute("*childraise-OnEnter", true);
	G.HeaderButton:SetAttribute("*childstate-OnEnter", "enter");
	G.HeaderButton:SetAttribute("*childstate-OnLeave", "leave");
    G.HeaderButton:SetAttribute("newstate1", "1-2");
    G.HeaderButton:SetAttribute("shift-newstate1", "1");
	
	G.SpellButtons = {};
	
	GFW_TrackMenu:SetupMenuOpenAction();
	
	G.HeaderButton:HookScript("OnEnter", G.OnButtonEnter);
	G.HeaderButton:HookScript("OnLeave", G.OnButtonLeave);
	G.HeaderButton:HookScript("OnClick", G.OnButtonLeave);

end

function GFW_TrackMenu:SetupMenuOpenAction()
	if (self.db.profile.open == "click") then
		G.RootFrame:SetAttribute("statemap-anchor-enter", nil);
	else						-- over
		G.RootFrame:SetAttribute("statemap-anchor-enter", "1:2");
	end
end

function GFW_TrackMenu:GetSpellButton(index)
    local b = G.SpellButtons[index];
    if (not b) then
		b = CreateFrame("Button", nil, nil, "FTM_ButtonTemplate");
		--b:SetWidth(150);
		--b:SetHeight(26);
		b:SetFrameStrata("DIALOG");
		b.texture = b:CreateTexture(nil, "OVERLAY");
		b.texture:SetWidth(24);
		b.texture:SetHeight(24);
		b.texture:SetPoint("LEFT", 4, 0);
		b.texture:Show();

		G.SpellButtons[index] = b;
		G.RootFrame:SetAttribute("addchild", b);
    end
    return b;
end


